use syn::parse::{Parse, ParseStream};
use syn::{Expr, Ident, LitStr, Token, custom_keyword};

custom_keyword!(transparent);

pub enum OsDisplayAttribute {
    Transparent,
    Format(FormatArgs),
}

pub struct FormatArgs {
    pub format_string: LitStr,
    pub positional_args: Vec<Expr>,
    pub named_args: Vec<(Ident, Expr)>,
}

impl Parse for OsDisplayAttribute {
    fn parse(input: ParseStream) -> syn::Result<Self> {
        let lookahead = input.lookahead1();

        if lookahead.peek(transparent) {
            input.parse::<transparent>()?;
            if !input.is_empty() {
                return Err(input.error("Unexpected tokens after `transparent` attribute."));
            }
            return Ok(OsDisplayAttribute::Transparent);
        } else if lookahead.peek(LitStr) {
            let format_string = input.parse()?;

            let mut positional_args = Vec::new();
            let mut named_args = Vec::new();

            while input.peek(Token![,]) {
                input.parse::<Token![,]>()?;

                if input.is_empty() {
                    break;
                }

                let lookahead_arg = input.fork();
                if lookahead_arg.peek(Ident) && lookahead_arg.peek2(Token![=]) {
                    let name: Ident = input.parse()?;
                    input.parse::<Token![=]>()?;
                    let value: Expr = input.parse()?;
                    named_args.push((name, value));
                } else {
                    let expr: Expr = input.parse()?;
                    positional_args.push(expr);
                }
            }

            Ok(OsDisplayAttribute::Format(FormatArgs {
                format_string,
                positional_args,
                named_args,
            }))
        } else {
            Err(lookahead.error())
        }
    }
}

impl Default for FormatArgs {
    fn default() -> Self {
        FormatArgs {
            format_string: LitStr::new("", proc_macro2::Span::call_site()),
            positional_args: Vec::new(),
            named_args: Vec::new(),
        }
    }
}
